precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;
uniform sampler2D inputImageTexture5;
uniform sampler2D inputImageTexture6;
uniform float intensity1;
uniform float intensity2;
uniform float intensity3;
uniform float v1;
uniform float v2;
uniform float v3;

vec3 blend_overlay_hardlight(vec4 base, vec4 overlay) {
    float ra;
    if (2.0 * overlay.r < overlay.a) {
        ra = 2.0 * overlay.r * base.r + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    } else {
        ra = overlay.a * base.a - 2.0 * (base.a - base.r) * (overlay.a - overlay.r) + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    }

    float ga;
    if (2.0 * overlay.g < overlay.a) {
        ga = 2.0 * overlay.g * base.g + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    } else {
        ga = overlay.a * base.a - 2.0 * (base.a - base.g) * (overlay.a - overlay.g) + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    }

    float ba;
    if (2.0 * overlay.b < overlay.a) {
        ba = 2.0 * overlay.b * base.b + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    } else {
        ba = overlay.a * base.a - 2.0 * (base.a - base.b) * (overlay.a - overlay.b) + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    }

    return vec3(ra, ga, ba);
}

vec4 blend_overlay(vec4 base, vec4 overlay) {
    return vec4(blend_overlay_hardlight(overlay, base), base.a + overlay.a - base.a * overlay.a);
}

highp vec3 rgb2hsv(vec3 c)    {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

highp vec3 hsv2rgb(vec3 c)    {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

highp vec3 blend(vec3 rgb, vec4 mask, vec4 highlight, vec4 origin, float intensity, float v) {
    vec3 hsl = rgb2hsv(rgb);
    hsl.z = hsl.z - hsl.z * (hsl.z - v) * mask.a * intensity;
    rgb = hsv2rgb(hsl);
    vec4 rrr = vec4(rgb, 1.0);

    vec4 tmp = blend_overlay(origin, vec4(1.0));
    rrr = mix(rrr, tmp, highlight.a * intensity * 0.5);
    return rrr.rgb;
}

void main() {
    vec4 c = texture2D(inputImageTexture, textureCoordinate);
    vec3 rgb = c.rgb;
    if (intensity1 > 0.001) {
        vec4 mask = texture2D(inputImageTexture1, textureCoordinate);
        vec4 highlight = texture2D(inputImageTexture2, textureCoordinate);
        rgb = blend(rgb, mask, highlight, c, intensity1, v1);
    }
    if (intensity2 > 0.001) {
        vec4 mask = texture2D(inputImageTexture3, textureCoordinate);
        vec4 highlight = texture2D(inputImageTexture4, textureCoordinate);
        rgb = blend(rgb, mask, highlight, c, intensity2, v2);
    }
    if (intensity3 > 0.001) {
        vec4 mask = texture2D(inputImageTexture5, textureCoordinate);
        vec4 highlight = texture2D(inputImageTexture6, textureCoordinate);
        rgb = blend(rgb, mask, highlight, c, intensity3, v3);
    }

    gl_FragColor = vec4(rgb, c.a);
}
