precision highp float;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;
uniform float intensity1;
uniform float intensity2;
uniform float intensity3;
uniform float intensity4;
uniform float v1;
uniform float v2;
uniform float v3;
uniform float v4;

highp vec3 rgb2hsv(highp vec3 c)    {
    highp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    highp vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    highp vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

highp vec3 hsv2rgb(vec3 c)    {
    highp vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    highp vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
    vec4 c = texture2D(inputImageTexture, textureCoordinate);
    vec3 hsv = rgb2hsv(c.rgb);
    if (intensity1 > 0.001) {
    vec4 mask = texture2D(inputImageTexture1, textureCoordinate);
    hsv.z = hsv.z - hsv.z * (hsv.z - v1) * mask.a * intensity1;
    }
    if (intensity2 > 0.001) {
    vec4 mask = texture2D(inputImageTexture2, textureCoordinate);
    hsv.z = hsv.z - hsv.z * (hsv.z - v2) * mask.a * intensity2;
    }
    if (intensity3 > 0.001) {
    vec4 mask = texture2D(inputImageTexture3, textureCoordinate);
    hsv.z = hsv.z - hsv.z * (hsv.z - v3) * mask.a * intensity3;
    }
    if (intensity4 > 0.001) {
    vec4 mask = texture2D(inputImageTexture4, textureCoordinate);
    hsv.z = hsv.z - hsv.z * (hsv.z - v4) * mask.a * intensity4;
    }

    vec3 rgb = hsv2rgb(hsv);
    gl_FragColor = vec4(rgb, c.a);
}
